@@ -7,6 +7,10 @@ import torch
 import torch.nn.functional as F
 from torch import nn, einsum
 from torch.utils.checkpoint import checkpoint
+
+from torch.autograd import Function
+import torch.distributed as distributed
+
 from einops import rearrange, repeat, reduce
 from einops.layers.torch import Rearrange, Reduce
 
@@ -40,11 +44,16 @@ def masked_mean(t, mask, dim = 1, eps = 1e-6):
     denom = mask.sum(dim = dim).clamp(min = eps)
     return numer / denom
 
+def pad_dim_to(t, length, dim = 0):
+    pad_length = length - t.shape[dim]
+    zero_pairs = (-dim - 1) if dim < 0 else (t.ndim - dim - 1)
+    return F.pad(t, (*((0, 0) * zero_pairs), 0, pad_length))
+
 def log(t, eps = 1e-20):
     return torch.log(t + eps)
 
 def l2norm(t):
-    return F.normalize(t, dim = -1, p = 2)
+    return F.normalize(t, dim = -1)
 
 def matrix_diag(t):
     device = t.device
@@ -56,17 +65,63 @@ def matrix_diag(t):
     diag_el = t.masked_select(diag_mask)
     return rearrange(diag_el, '(b d) -> b d', d = num_diag_el)
 
+# distributed helpers
+
+def all_gather_variable_batch(t, dim = 0):
+    device, rank, world_size = t.device, distributed.get_rank(), distributed.get_world_size()
+
+    size = torch.tensor(t.shape[dim], device = device, dtype = torch.long)
+    sizes = [torch.empty_like(size, device = device, dtype = torch.long) for i in range(world_size)]
+    distributed.all_gather(sizes, size)
+
+    sizes = torch.stack(sizes)
+    max_size = sizes.amax().item()
+    padded_t = pad_dim_to(t, max_size, dim = dim)
+
+    gathered_tensors = [torch.empty(padded_t.shape, device = device, dtype = padded_t.dtype) for i in range(world_size)]
+    distributed.all_gather(gathered_tensors, padded_t)
+
+    gathered_tensor = torch.cat(gathered_tensors, dim = dim)
+    seq = torch.arange(max_size, device = device)
+
+    mask = rearrange(seq, 'j -> 1 j') < rearrange(sizes, 'i -> i 1')
+    mask = rearrange(mask, 'i j -> (i j)')
+    seq = torch.arange(mask.shape[-1], device = device)
+    indices = seq[mask]
+
+    gathered_tensor = gathered_tensor.index_select(dim, indices)
+    sizes = sizes.tolist()
+
+    return gathered_tensor, sizes
+
+class MaybeAllGather(Function):
+    @staticmethod
+    def forward(ctx, x, dim):
+        is_distributed = distributed.is_initialized() and distributed.get_world_size() > 1
+        ctx.is_distributed = is_distributed
+        ctx.dim = dim
+
+        if not is_distributed:
+            return x
+
+        x, batch_sizes = all_gather_variable_dim(x, dim = dim)
+        ctx.batch_sizes = batch_sizes
+        return x
+
+    @staticmethod
+    def backward(ctx, grads):
+        if not ctx.is_distributed:
+            return grads, None
+
+        batch_sizes, rank = ctx.batch_sizes, distributed.get_rank()
+        grads_by_rank = grads.split(batch_sizes, dim = ctx.dim)
+        return grads_by_rank[rank], None
+
+maybe_all_gather = MaybeAllGather.apply
+
 # checkpointing helper function
 
-def make_checkpointable(fn, **kwargs):
-    if isinstance(fn, nn.ModuleList):
-        return [maybe(make_checkpointable)(el, **kwargs) for el in fn]
-
-    condition = kwargs.pop('condition', None)
-
-    if exists(condition) and not condition(fn):
-        return fn
-
+def make_checkpointable(fn):
     @wraps(fn)
     def inner(*args):
         input_needs_grad = any([isinstance(el, torch.Tensor) and el.requires_grad for el in args])
@@ -129,6 +184,27 @@ class PreNorm(nn.Module):
     def forward(self, x, *args, **kwargs):
         return self.fn(self.norm(x), *args, **kwargs)
 
+# patch dropout
+
+class PatchDropout(nn.Module):
+    def __init__(self, prob):
+        super().__init__()
+        assert 0 <= prob < 1.
+        self.prob = prob
+
+    def forward(self, x, force_keep_all = False):
+        if not self.training or self.prob == 0. or force_keep_all:
+            return x
+
+        b, n, _, device = *x.shape, x.device
+
+        batch_indices = torch.arange(b, device = device)
+        batch_indices = rearrange(batch_indices, '... -> ... 1')
+        num_patches_keep = max(1, int(n * (1 - self.prob)))
+        patch_indices_keep = torch.randn(b, n, device = device).topk(num_patches_keep, dim = -1).indices
+
+        return x[batch_indices, patch_indices_keep]
+
 # rotary positional embedding
 
 class RotaryEmbedding(nn.Module):
@@ -178,9 +254,10 @@ class FeedForward(nn.Module):
         return self.net(x)
 
 class Attention(nn.Module):
-    def __init__(self, dim, dim_head = 64, heads = 8, dropout = 0.):
+    def __init__(self, dim, dim_head = 64, heads = 8, causal = False, dropout = 0.):
         super().__init__()
         self.heads = heads
+        self.causal = causal
         self.scale = dim_head ** -0.5
         inner_dim = dim_head * heads
 
@@ -189,7 +266,8 @@ class Attention(nn.Module):
         self.dropout = nn.Dropout(dropout)
 
     def forward(self, x, mask = None, rotary_pos_emb = None):
-        h = self.heads
+        h, device, scale = self.heads, x.device, self.scale
+
         q, k, v = self.to_qkv(x).chunk(3, dim = -1)
         q, k, v = map(lambda t: rearrange(t, 'b n (h d) -> b h n d', h = h), (q, k, v))
 
@@ -201,12 +279,20 @@ class Attention(nn.Module):
 
         sim = einsum('b h i d, b h j d -> b h i j', q, k)
 
+        mask_value = -torch.finfo(sim.dtype).max
+
         if exists(mask):
             mask = rearrange(mask, 'b j -> b 1 1 j')
-            mask_value = -torch.finfo(sim.dtype).max
             sim = sim.masked_fill(~mask, mask_value)
 
+        if self.causal:
+            i, j = sim.shape[-2:]
+            causal_mask = torch.ones((i, j), dtype = torch.bool, device = device).triu(j - i + 1)
+            sim = sim.masked_fill(causal_mask, mask_value)
+
         attn = sim.softmax(dim = -1, dtype = torch.float32)
+        attn = attn.type(sim.dtype)
+
         attn = self.dropout(attn)
 
         out = einsum('b h i j, b h j d -> b h i d', attn, v)
@@ -221,6 +307,7 @@ class Transformer(nn.Module):
         depth,
         dim_head = 64,
         heads = 8,
+        causal = False,
         attn_dropout = 0.,
         ff_dropout = 0.,
         ff_mult = 4,
@@ -232,7 +319,7 @@ class Transformer(nn.Module):
         self.layers = nn.ModuleList([])
         for _ in range(depth):
             self.layers.append(nn.ModuleList([
-                PreNorm(dim, Attention(dim = dim, dim_head = dim_head, heads = heads, dropout = attn_dropout)),
+                PreNorm(dim, Attention(dim = dim, dim_head = dim_head, heads = heads, causal = causal, dropout = attn_dropout)),
                 PreNorm(dim, FeedForward(dim = dim, mult = ff_mult)),
             ]))
 
@@ -269,6 +356,7 @@ class TextTransformer(nn.Module):
         max_seq_len,
         dim_head,
         rotary_pos_emb = None,
+        causal = False,
         **kwargs
     ):
         super().__init__()
@@ -277,9 +365,9 @@ class TextTransformer(nn.Module):
         self.abs_pos_emb = nn.Embedding(max_seq_len, dim) if not rotary_pos_emb else None
         self.rotary_pos_emb = RotaryEmbedding(min(dim_head, 32)) if rotary_pos_emb else None
 
-        self.cls_token = nn.Parameter(torch.randn(dim))
+        self.cls_token = nn.Parameter(torch.randn(dim)) if not causal else None
 
-        self.transformer = Transformer(dim, dim_head = dim_head, **kwargs)
+        self.transformer = Transformer(dim, dim_head = dim_head, causal = causal, **kwargs)
 
     def forward(self, x, mask = None):
         b, n, device = *x.shape, x.device
@@ -294,11 +382,12 @@ class TextTransformer(nn.Module):
         if exists(self.rotary_pos_emb):
             rotary_pos_emb = self.rotary_pos_emb(n + 1, device = device)
 
-        cls_tokens = repeat(self.cls_token, 'd -> b 1 d', b = b)
-        x = torch.cat((cls_tokens, x), dim = 1)
+        if exists(self.cls_token):
+            cls_tokens = repeat(self.cls_token, 'd -> b 1 d', b = b)
+            x = torch.cat((cls_tokens, x), dim = 1)
 
-        if exists(mask):
-            mask = F.pad(mask, (1, 0), value = True)
+            if exists(mask):
+                mask = F.pad(mask, (1, 0), value = True)
 
         out = self.transformer(x, mask = mask, rotary_pos_emb = rotary_pos_emb)
         return out
@@ -311,6 +400,7 @@ class VisionTransformer(nn.Module):
         image_size,
         patch_size,
         channels,
+        patch_dropout = 0.5,
         **kwargs
     ):
         super().__init__()
@@ -324,6 +414,8 @@ class VisionTransformer(nn.Module):
         )
 
         self.pos_emb = nn.Embedding(num_patches, dim)
+        self.patch_dropout = PatchDropout(patch_dropout)
+
         self.transformer = Transformer(dim, **kwargs)
 
         self.to_cls_tokens = nn.Sequential(
@@ -332,7 +424,11 @@ class VisionTransformer(nn.Module):
             Rearrange('b d -> b 1 d')
         )
 
-    def forward(self, x):
+    def forward(
+        self,
+        x,
+        keep_all_patches = False
+    ):
         device = x.device
 
         x = self.to_tokens(x)
@@ -341,6 +437,8 @@ class VisionTransformer(nn.Module):
         pos_emb = self.pos_emb(torch.arange(n, device = device))
         x = x + rearrange(pos_emb, 'n d -> 1 n d')
 
+        x = self.patch_dropout(x, force_keep_all = keep_all_patches)
+
         out = self.transformer(x)
 
         cls_tokens = self.to_cls_tokens(out)
@@ -383,12 +481,15 @@ class CLIP(nn.Module):
         text_has_cls_token = True,
         text_pad_id = 0,
         text_rotary_pos_emb = False,
+        text_causal_mask = False,
+        text_eos_id = None,
         text_encode_without_mask = False,
         visual_enc_depth = 6,
         visual_heads = 8,
         visual_dim_head = 64,
         visual_image_size = 256,
         visual_patch_size = 32,
+        visual_patch_dropout = 0.5,
         visual_has_cls_token = True,
         channels = 3,
         use_all_token_embeds = False,
@@ -427,6 +528,11 @@ class CLIP(nn.Module):
 
         self.text_encode_without_mask = text_encode_without_mask # whether to pass in text mask to text encoder
 
+        self.text_causal_mask = text_causal_mask
+        self.text_eos_id = text_eos_id
+
+        assert not (text_causal_mask and not exists(text_eos_id)), 'text EOS token id must be given if using causal mask in text transformer'
+
         if exists(text_encoder):
             self.text_transformer = text_encoder
         else:
@@ -436,6 +542,7 @@ class CLIP(nn.Module):
                 max_seq_len = text_seq_len,
                 depth = text_enc_depth,
                 heads = text_heads,
+                causal = text_causal_mask,
                 dim_head = text_dim_head,
                 rotary_pos_emb = text_rotary_pos_emb,
                 checkpoint_during_training = checkpoint_during_training
@@ -456,6 +563,7 @@ class CLIP(nn.Module):
                 depth = visual_enc_depth,
                 heads = visual_heads,
                 dim_head = visual_dim_head,
+                patch_dropout = visual_patch_dropout,
                 checkpoint_during_training = checkpoint_during_training
             )
 
@@ -604,6 +712,25 @@ class CLIP(nn.Module):
             freeze = freeze_text_encoder
         )
 
+        # depending on whether text is using causal mask, post process, moving eos token to the first position
+
+        if self.text_causal_mask:
+            eos_text_mask = (text == self.text_eos_id)
+            assert torch.all(torch.any(eos_text_mask, dim = -1)), f'some of the text rows does not have the eos id {self.text_eos_id}'
+
+            text_len = text.shape[-1]
+            eos_indices = eos_text_mask.float().argmax(dim = -1, keepdim = True)
+
+            eos_text_mask = torch.zeros_like(eos_text_mask).scatter(1, eos_indices, 1.).bool()
+            eos_text_mask = rearrange(eos_text_mask, '... -> ... 1')
+
+            eos_tokens = enc_text.masked_select(eos_text_mask)
+            rest_tokens = enc_text.masked_select(~eos_text_mask)
+
+            eos_tokens = rearrange(eos_tokens, '(b d) -> b 1 d', b = b)
+            rest_tokens = rearrange(rest_tokens, '(b n d) -> b n d', b = b, n = text_len - 1)
+            enc_text = torch.cat((eos_tokens, rest_tokens), dim = 1)
+
         # whether to train image encoder, in the case that the image net was pretrained as recommended in LiT
 
         enc_image = model_forward_with_context(
@@ -612,11 +739,6 @@ class CLIP(nn.Module):
             freeze = freeze_image_encoder
         )
 
-        # asserts
-
-        assert enc_text.ndim == 3, 'encoded text must have 3 dimensions (batch, seq, features)'
-        assert enc_image.ndim == 3, 'encoded image must have 3 dimensions (batch, seq [height x width], features)'
-
         # early return of encodings, if needed (for DALL-E2)
 
         if return_encodings:
@@ -625,11 +747,13 @@ class CLIP(nn.Module):
         # depending on whether to do fine-grained CLIP or not, select either all tokens, or CLS tokens only
 
         if self.use_all_token_embeds:
+            assert enc_text.ndim == 3, 'encoded text must have 3 dimensions (batch, seq, features)'
+            assert enc_image.ndim == 3, 'encoded image must have 3 dimensions (batch, seq [height x width], features)'
             text_embeds = enc_text[:, 1:] if self.text_has_cls_token else enc_text
             image_embeds = enc_image[:, 1:] if self.visual_has_cls_token else enc_image
         else:
-            text_embeds = enc_text[:, 0]
-            image_embeds = enc_image[:, 0]
+            text_embeds = enc_text[:, 0] if enc_text.ndim == 3 else enc_text
+            image_embeds = enc_image[:, 0] if enc_image.ndim == 3 else enc_image
 
         # project to latents
 
@@ -677,6 +801,15 @@ class CLIP(nn.Module):
             text_latents_extra = rearrange(text_latents_extra, '(m b) ... -> m b ...', m = num_batch_texts)
             image_latents_extra = rearrange(image_latents_extra, '(m b) ... -> m b ...', m = num_batch_images)
 
+        # maybe distributed all gather
+
+        text_latents = maybe_all_gather(text_latents, 1)
+        image_latents = maybe_all_gather(image_latents, 1)
+
+        if self.extra_latent_projection:
+            text_latents_extra = maybe_all_gather(text_latents_extra, 1)
+            image_latents_extra = maybe_all_gather(image_latents_extra, 1)
+
         # contrastive loss
 
         """
@@ -733,8 +866,8 @@ class CLIP(nn.Module):
 
         # loss
 
-        text_to_image_loss = -log(text_to_image_pos / text_to_image_denom).mean(dim = -1)
-        image_to_text_loss = -log(image_to_text_pos / image_to_text_denom).mean(dim = -1)
+        text_to_image_loss = (-log(text_to_image_pos) + log(text_to_image_denom)).mean(dim = -1)
+        image_to_text_loss = (-log(image_to_text_pos) + log(image_to_text_denom)).mean(dim = -1)
 
         # calculate CL loss
 
